/* == PRIVATE VARIABLES == */

var _waitModes = {};          // Holds all wait modes
var _messageCodeConvert = []; // Holds all message codes on convert
var _messageCodeProcess = []; // Holds all message codes on process

/* == PRIVATE FUNCTIONS == */

/* Function for message codes */
var _setMessageCode = function(code) {
    code = code.replace(/\[(.+?)\]/, '\[$1\]');
    code = code.replace(/\:any/,'(.+?)');
    code = code.replace(/\:num/,'([0-9]+)');
    code = code.replace(/\:id/,'([0-9a-zA-Z_])');
    return code;
}

var _runMessageCode = function(func) {
    func.apply(this, Array.prototype.slice.call(arguments, 2));
}

/* == ALIASES == */

_old.gameInterpreterUpdateWaitMode = Game_Interpreter.prototype.updateWaitMode;
_old.windowBaseConvertEscapeCharacters = Window_Base.prototype.convertEscapeCharacters;
_old.windowBaseProcessEscapeCharacter = Window_Base.prototype.processEscapeCharacter;

/* == PLUGIN METHODS == */

/**
 * Easy function to add new wait modes
 */
/* NAMESPACE */addWaitMode = function(waitMode, waitFunc) {
    _waitModes[waitMode] = waitFunc;
}

/**
 * Easy function to add new message codes
 */
/* NAMESPACE */addMessageCode = function(code, func, type) {
    var data = [_setMessageCode(code), func];
    switch (type) {
        case 'process':
            _messageCodeProcess.push(data);
            break;
        default:
            _messageCodeConvert.push(data);
            break;
    }
}

/* NAMESPACE */addWaitMode.version = '1.0';
/* NAMESPACE */addMessageCode.version = '1.0';

/* == PLUGIN OVERRIDES == */

/* --------------------------------------------------------------------
 * - Game_Interpereter.prototype.updateWaitMode (Override)            -
 * --------------------------------------------------------------------
 */

Game_Interpreter.prototype.updateWaitMode = function() {
    if(_waitModes.hasOwnProperty(this._waitMode)) {
        var waiting = _waitModes[this._waitMode].call(this);
        if(!waiting) {
            this._waitMode = '';
        }
        return waiting;
    } else {
        return _old.gameInterpreterUpdateWaitMode.call(this);
    }
}

/* --------------------------------------------------------------------
 * - Window_Base.prototype.convertEscapeCharacters (Override)         -
 * --------------------------------------------------------------------
 */

Window_Base.prototype.convertEscapeCharacters = function(text) {
    text = _old.windowBaseConvertEscapeCharacters.call(this, text);
    for(var idx = 0; idx < _messageCodeConvert.length; idx++) {
        var data = _messageCodeConvert[idx];
        text = text.replace(new RegExp('\x1b' + data[0], 'gi'), _runMessageCode.bind(this, data[1]));
    }
    return text;
}

/* --------------------------------------------------------------------
 * - Window_Base.prototype.processEscapeCharacter (Override)          -
 * --------------------------------------------------------------------
 */

Window_Base.prototype.processEscapeCharacter = function(code, textState) {
    textState.index--;
    var text = textState.text.slice(textState.index);
    var found = false;
    for(var idx = 0; idx < _messageCodeProcess.length; idx++) {
        var data = _messageCodeProcess[idx];
        var regExp = new RegExp('^' + data[0], 'i');
        if(regExp.test(text)) {
            var args = text.match(regExp);
            var matched = args[0];
            args.unshift(data[1]);
            _runMessageCode.apply(this, args);
            textState.index+= matched.length;
            found = true;
            break;
        }
    }
    if (!found) {
        textState.index++;
        _old.windowBaseConvertEscapeCharacters.call(this, code, textState);
    }
}